"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Settled = exports.Rejected = exports.Fulfilled = exports.Pending = exports.Initial = void 0;

var _propTypes = _interopRequireDefault(require("./propTypes.js"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var nullify = function nullify(children) {
  return children === undefined ? null : children;
};

var renderFn = function renderFn(children) {
  for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    args[_key - 1] = arguments[_key];
  }

  return nullify(typeof children === "function" ? children.apply(void 0, args) : children);
};
/**
 * Renders only when no promise has started or completed yet.
 *
 * @prop {Function|Node} children Function (passing state) or React node
 * @prop {Object} state React Async state object
 * @prop {boolean} persist Show until we have data, even while pending (loading) or when an error occurred
 */


var Initial = function Initial(_ref) {
  var children = _ref.children,
      persist = _ref.persist,
      state = _ref.state;
  return state.isInitial || persist && !state.data ? renderFn(children, state) : null;
};
/**
 * Renders only while pending (promise is loading).
 *
 * @prop {Function|Node} children Function (passing state) or React node
 * @prop {Object} state React Async state object
 * @prop {boolean} initial Show only on initial load (data is undefined)
 */


exports.Initial = Initial;

var Pending = function Pending(_ref2) {
  var children = _ref2.children,
      initial = _ref2.initial,
      state = _ref2.state;
  return state.isPending && (!initial || !state.value) ? renderFn(children, state) : null;
};
/**
 * Renders only when promise is resolved.
 *
 * @prop {Function|Node} children Function (passing data and state) or React node
 * @prop {Object} state React Async state object
 * @prop {boolean} persist Show old data while pending (promise is loading)
 */


exports.Pending = Pending;

var Fulfilled = function Fulfilled(_ref3) {
  var children = _ref3.children,
      persist = _ref3.persist,
      state = _ref3.state;
  return state.isFulfilled || persist && state.data ? renderFn(children, state.data, state) : null;
};
/**
 * Renders only when promise is rejected.
 *
 * @prop {Function|Node} children Function (passing error and state) or React node
 * @prop {Object} state React Async state object
 * @prop {boolean} persist Show old error while pending (promise is loading)
 */


exports.Fulfilled = Fulfilled;

var Rejected = function Rejected(_ref4) {
  var children = _ref4.children,
      persist = _ref4.persist,
      state = _ref4.state;
  return state.isRejected || persist && state.error ? renderFn(children, state.error, state) : null;
};
/**
 * Renders only when promise is fulfilled or rejected.
 *
 * @prop {Function|Node} children Function (passing state) or React node
 * @prop {Object} state React Async state object
 * @prop {boolean} persist Show old data or error while pending (promise is loading)
 */


exports.Rejected = Rejected;

var Settled = function Settled(_ref5) {
  var children = _ref5.children,
      persist = _ref5.persist,
      state = _ref5.state;
  return state.isSettled || persist && state.value ? renderFn(children, state) : null;
};

exports.Settled = Settled;

if (_propTypes["default"]) {
  Initial.propTypes = _propTypes["default"].Initial;
  Pending.propTypes = _propTypes["default"].Pending;
  Fulfilled.propTypes = _propTypes["default"].Fulfilled;
  Rejected.propTypes = _propTypes["default"].Rejected;
  Settled.propTypes = _propTypes["default"].Settled;
}