"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.dispatchMiddleware = exports.reducer = exports.init = exports.actionTypes = void 0;

var _status = require("./status.js");

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var actionTypes = {
  start: "start",
  cancel: "cancel",
  fulfill: "fulfill",
  reject: "reject"
};
exports.actionTypes = actionTypes;

var init = function init(_ref) {
  var initialValue = _ref.initialValue,
      promise = _ref.promise,
      promiseFn = _ref.promiseFn;
  return _objectSpread({
    initialValue: initialValue,
    data: initialValue instanceof Error ? undefined : initialValue,
    error: initialValue instanceof Error ? initialValue : undefined,
    value: initialValue,
    startedAt: promise || promiseFn ? new Date() : undefined,
    finishedAt: initialValue ? new Date() : undefined
  }, (0, _status.getStatusProps)((0, _status.getInitialStatus)(initialValue, promise || promiseFn)), {
    counter: 0
  });
};

exports.init = init;

var reducer = function reducer(state, _ref2) {
  var type = _ref2.type,
      payload = _ref2.payload,
      meta = _ref2.meta;

  switch (type) {
    case actionTypes.start:
      return _objectSpread({}, state, {
        startedAt: new Date(),
        finishedAt: undefined
      }, (0, _status.getStatusProps)(_status.statusTypes.pending), {
        counter: meta.counter
      });

    case actionTypes.cancel:
      return _objectSpread({}, state, {
        startedAt: undefined,
        finishedAt: undefined
      }, (0, _status.getStatusProps)((0, _status.getIdleStatus)(state.error || state.data)), {
        counter: meta.counter
      });

    case actionTypes.fulfill:
      return _objectSpread({}, state, {
        data: payload,
        value: payload,
        error: undefined,
        finishedAt: new Date()
      }, (0, _status.getStatusProps)(_status.statusTypes.fulfilled));

    case actionTypes.reject:
      return _objectSpread({}, state, {
        error: payload,
        value: payload,
        finishedAt: new Date()
      }, (0, _status.getStatusProps)(_status.statusTypes.rejected));

    default:
      return state;
  }
};

exports.reducer = reducer;

var dispatchMiddleware = function dispatchMiddleware(dispatch) {
  return function (action) {
    for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }

    dispatch.apply(void 0, [action].concat(args));

    if (action.type === actionTypes.start && typeof action.payload === "function") {
      action.payload();
    }
  };
};

exports.dispatchMiddleware = dispatchMiddleware;