"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getStatusProps = exports.getIdleStatus = exports.getInitialStatus = exports.statusTypes = void 0;
var statusTypes = {
  initial: "initial",
  pending: "pending",
  fulfilled: "fulfilled",
  rejected: "rejected"
};
exports.statusTypes = statusTypes;

var getInitialStatus = function getInitialStatus(value, promise) {
  if (value instanceof Error) return statusTypes.rejected;
  if (value !== undefined) return statusTypes.fulfilled;
  if (promise) return statusTypes.pending;
  return statusTypes.initial;
};

exports.getInitialStatus = getInitialStatus;

var getIdleStatus = function getIdleStatus(value) {
  if (value instanceof Error) return statusTypes.rejected;
  if (value !== undefined) return statusTypes.fulfilled;
  return statusTypes.initial;
};

exports.getIdleStatus = getIdleStatus;

var getStatusProps = function getStatusProps(status) {
  return {
    status: status,
    isInitial: status === statusTypes.initial,
    isPending: status === statusTypes.pending,
    isLoading: status === statusTypes.pending,
    // alias
    isFulfilled: status === statusTypes.fulfilled,
    isResolved: status === statusTypes.fulfilled,
    // alias
    isRejected: status === statusTypes.rejected,
    isSettled: status === statusTypes.fulfilled || status === statusTypes.rejected
  };
};

exports.getStatusProps = getStatusProps;