"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useFetch = exports["default"] = void 0;

var _react = require("react");

var _reducer = require("./reducer.js");

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var noop = function noop() {};

var useAsync = function useAsync(arg1, arg2) {
  var options = typeof arg1 === "function" ? _objectSpread({}, arg2, {
    promiseFn: arg1
  }) : arg1;
  var counter = (0, _react.useRef)(0);
  var isMounted = (0, _react.useRef)(true);
  var lastArgs = (0, _react.useRef)(undefined);
  var prevOptions = (0, _react.useRef)(undefined);
  var abortController = (0, _react.useRef)({
    abort: noop
  });

  var _ref = window.__REACT_ASYNC__ || {},
      devToolsDispatcher = _ref.devToolsDispatcher;

  var reducer = options.reducer,
      _options$dispatcher = options.dispatcher,
      dispatcher = _options$dispatcher === void 0 ? devToolsDispatcher : _options$dispatcher;

  var _useReducer = (0, _react.useReducer)(reducer ? function (state, action) {
    return reducer(state, action, _reducer.reducer);
  } : _reducer.reducer, options, _reducer.init),
      _useReducer2 = _slicedToArray(_useReducer, 2),
      state = _useReducer2[0],
      _dispatch = _useReducer2[1];

  var dispatch = dispatcher ? function (action) {
    return dispatcher(action, (0, _reducer.dispatchMiddleware)(_dispatch), options);
  } : (0, _reducer.dispatchMiddleware)(_dispatch);

  var getMeta = function getMeta(meta) {
    return _objectSpread({
      counter: counter.current,
      debugLabel: options.debugLabel
    }, meta);
  };

  var setData = function setData(data) {
    var callback = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : noop;

    if (isMounted.current) {
      dispatch({
        type: _reducer.actionTypes.fulfill,
        payload: data,
        meta: getMeta()
      });
      callback();
    }

    return data;
  };

  var setError = function setError(error) {
    var callback = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : noop;

    if (isMounted.current) {
      dispatch({
        type: _reducer.actionTypes.reject,
        payload: error,
        error: true,
        meta: getMeta()
      });
      callback();
    }

    return error;
  };

  var onResolve = options.onResolve,
      onReject = options.onReject;

  var handleResolve = function handleResolve(count) {
    return function (data) {
      return count === counter.current && setData(data, function () {
        return onResolve && onResolve(data);
      });
    };
  };

  var handleReject = function handleReject(count) {
    return function (error) {
      return count === counter.current && setError(error, function () {
        return onReject && onReject(error);
      });
    };
  };

  var start = function start(promiseFn) {
    if ("AbortController" in window) {
      abortController.current.abort();
      abortController.current = new window.AbortController();
    }

    counter.current++;
    return new Promise(function (resolve, reject) {
      if (!isMounted.current) return;

      var executor = function executor() {
        return promiseFn().then(resolve, reject);
      };

      dispatch({
        type: _reducer.actionTypes.start,
        payload: executor,
        meta: getMeta()
      });
    });
  };

  var promise = options.promise,
      promiseFn = options.promiseFn,
      initialValue = options.initialValue;

  var load = function load() {
    if (promise) {
      return start(function () {
        return promise;
      }).then(handleResolve(counter.current), handleReject(counter.current));
    }

    var isPreInitialized = initialValue && counter.current === 0;

    if (promiseFn && !isPreInitialized) {
      return start(function () {
        return promiseFn(options, abortController.current);
      }).then(handleResolve(counter.current), handleReject(counter.current));
    }
  };

  var deferFn = options.deferFn;

  var run = function run() {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    if (deferFn) {
      lastArgs.current = args;
      return start(function () {
        return deferFn(args, options, abortController.current);
      }).then(handleResolve(counter.current), handleReject(counter.current));
    }
  };

  var cancel = function cancel() {
    counter.current++;
    abortController.current.abort();
    isMounted.current && dispatch({
      type: _reducer.actionTypes.cancel,
      meta: getMeta()
    });
  };

  var watch = options.watch,
      watchFn = options.watchFn;
  (0, _react.useEffect)(function () {
    if (watchFn && prevOptions.current && watchFn(options, prevOptions.current)) load();
  });
  (0, _react.useEffect)(function () {
    promise || promiseFn ? load() : cancel();
  }, [promise, promiseFn, watch]);
  (0, _react.useEffect)(function () {
    return function () {
      return isMounted.current = false;
    };
  }, []);
  (0, _react.useEffect)(function () {
    return function () {
      return abortController.current.abort();
    };
  }, []);
  (0, _react.useEffect)(function () {
    return (prevOptions.current = options) && undefined;
  });
  (0, _react.useDebugValue)(state, function (_ref2) {
    var status = _ref2.status;
    return "[".concat(counter.current, "] ").concat(status);
  });
  return (0, _react.useMemo)(function () {
    return _objectSpread({}, state, {
      cancel: cancel,
      run: run,
      reload: function reload() {
        return lastArgs.current ? run.apply(void 0, _toConsumableArray(lastArgs.current)) : load();
      },
      setData: setData,
      setError: setError
    });
  }, [state, deferFn, onResolve, onReject, dispatcher, reducer]);
};

var parseResponse = function parseResponse(accept, json) {
  return function (res) {
    if (!res.ok) return Promise.reject(res);
    if (json === false) return res;
    if (json === true || accept === "application/json") return res.json();
    return res;
  };
};

var useAsyncFetch = function useAsyncFetch(input, init) {
  var _ref3 = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  var defer = _ref3.defer,
      json = _ref3.json,
      options = _objectWithoutProperties(_ref3, ["defer", "json"]);

  var method = input.method || init && init.method;
  var headers = input.headers || init && init.headers || {};
  var accept = headers["Accept"] || headers["accept"] || headers.get && headers.get("accept");

  var doFetch = function doFetch(input, init) {
    return window.fetch(input, init).then(parseResponse(accept, json));
  };

  var isDefer = defer === true || ~["POST", "PUT", "PATCH", "DELETE"].indexOf(method);
  var fn = defer === false || !isDefer ? "promiseFn" : "deferFn";
  var state = useAsync(_objectSpread({}, options, _defineProperty({}, fn, (0, _react.useCallback)(function (_, props, ctrl) {
    return doFetch(input, _objectSpread({
      signal: ctrl ? ctrl.signal : props.signal
    }, init));
  }, [JSON.stringify(input), JSON.stringify(init)]))));
  (0, _react.useDebugValue)(state, function (_ref4) {
    var counter = _ref4.counter,
        status = _ref4.status;
    return "[".concat(counter, "] ").concat(status);
  });
  return state;
};

var unsupported = function unsupported() {
  throw new Error("useAsync requires React v16.8 or up. Upgrade your React version or use the <Async> component instead.");
};

var _default = _react.useEffect ? useAsync : unsupported;

exports["default"] = _default;
var useFetch = _react.useEffect ? useAsyncFetch : unsupported;
exports.useFetch = useFetch;